import Flutter
import UIKit
import Photos

public class SwiftImageGallerySaverPlugin: NSObject, FlutterPlugin {
    var result: FlutterResult?;

    public static func register(with registrar: FlutterPluginRegistrar) {
      let channel = FlutterMethodChannel(name: "image_gallery_saver", binaryMessenger: registrar.messenger())
      let instance = SwiftImageGallerySaverPlugin()
      registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
      self.result = result
      if call.method == "saveImageToGallery" {
        let arguments = call.arguments as? [String: Any] ?? [String: Any]()
        guard let imageData = (arguments["imageBytes"] as? FlutterStandardTypedData)?.data,
            let image = UIImage(data: imageData),
            let quality = arguments["quality"] as? Int ,
            let _ = arguments["name"]
            else { return }
        let newImage = image.jpegData(compressionQuality: CGFloat(quality / 100))!
        saveImage(UIImage(data: newImage) ?? image)
      } else if (call.method == "saveFileToGallery") {
        guard let path = call.arguments as? String else { return }
        if (isImageFile(filename: path)) {
            saveImageAtFileUrl(path)
        } else {
            if (UIVideoAtPathIsCompatibleWithSavedPhotosAlbum(path)) {
                saveVideo(path)
            }
        }
      } else {
        result(FlutterMethodNotImplemented)
      }
    }
    
    func saveVideo(_ path: String) {
        var videoIds: [String] = []
        
        PHPhotoLibrary.shared().performChanges {
            let req = PHAssetChangeRequest.creationRequestForAssetFromVideo(atFileURL: URL(string: path)!)
            if let videoId = req?.placeholderForCreatedAsset?.localIdentifier {
                videoIds.append(videoId)
            }
        } completionHandler: { [unowned self] (success, error) in
            DispatchQueue.main.async {
                if (success) {
                    let assetResult = PHAsset.fetchAssets(withLocalIdentifiers: videoIds, options: nil)
                    let videoAsset = assetResult[0]
                    PHImageManager().requestAVAsset(forVideo: videoAsset, options: nil) { (avurlAsset, audioMix, info) in
                        if let urlStr = (avurlAsset as? AVURLAsset)?.url.absoluteString {
                            self.result?(urlStr)
                        }
                    }
                }
            }
        }
    }
    
    func saveImage(_ image: UIImage) {
        var imageIds: [String] = []
        
        PHPhotoLibrary.shared().performChanges {
            let req = PHAssetChangeRequest.creationRequestForAsset(from: image)
            if let imageId = req.placeholderForCreatedAsset?.localIdentifier {
                imageIds.append(imageId)
            }
        } completionHandler: { [unowned self] (success, error) in
            DispatchQueue.main.async {
                if (success) {
                    let assetResult = PHAsset.fetchAssets(withLocalIdentifiers: imageIds, options: nil)
                    let imageAsset = assetResult[0]
                    let options = PHContentEditingInputRequestOptions()
                    options.canHandleAdjustmentData = { (adjustmeta)
                        -> Bool in true }
                    imageAsset.requestContentEditingInput(with: options) { [unowned self] (contentEditingInput, info) in
                        if let urlStr = contentEditingInput?.fullSizeImageURL?.absoluteString {
                            self.result?(urlStr)
                        }
                    }
                    
                }
            }
        }
    }
    
    func saveImageAtFileUrl(_ url: String) {
        var imageIds: [String] = []
        
        PHPhotoLibrary.shared().performChanges {
            let req = PHAssetChangeRequest.creationRequestForAssetFromImage(atFileURL: URL(string: url)!)
            if let imageId = req?.placeholderForCreatedAsset?.localIdentifier {
                imageIds.append(imageId)
            }
        } completionHandler: { [unowned self] (success, error) in
            DispatchQueue.main.async {
                if (success) {
                    let assetResult = PHAsset.fetchAssets(withLocalIdentifiers: imageIds, options: nil)
                    let imageAsset = assetResult[0]
                    let options = PHContentEditingInputRequestOptions()
                    options.canHandleAdjustmentData = { (adjustmeta)
                        -> Bool in true }
                    imageAsset.requestContentEditingInput(with: options) { [unowned self] (contentEditingInput, info) in
                        if let urlStr = contentEditingInput?.fullSizeImageURL?.absoluteString {
                            self.result?(urlStr)
                        }
                    }
                    
                }
            }
        }
    }

    /// finish saving，if has error，parameters error will not nill
    @objc func didFinishSavingImage(image: UIImage, error: NSError?, contextInfo: UnsafeMutableRawPointer?) {
        result?(error == nil)
    }
    
    @objc func didFinishSavingVideo(videoPath: String, error: NSError?, contextInfo: UnsafeMutableRawPointer?) {
        result?(error == nil)
    }
    
    func isImageFile(filename: String) -> Bool {
        return filename.hasSuffix(".jpg")
            || filename.hasSuffix(".png")
            || filename.hasSuffix(".JPEG")
            || filename.hasSuffix(".JPG")
            || filename.hasSuffix(".PNG")
            || filename.hasSuffix(".gif")
            || filename.hasSuffix(".GIF")
    }
}
